from django import template
from django.conf import settings
from django.urls import reverse

from wagtail_bynder import get_video_model


register = template.Library()


@register.simple_tag
def bynder_domain():
    return getattr(settings, "BYNDER_DOMAIN", "")


@register.simple_tag
def bynder_compactview_api_token():
    return getattr(settings, "BYNDER_COMPACTVIEW_API_TOKEN", "")


@register.simple_tag
def get_image_chosen_base_url():
    url = reverse("wagtailimages_chooser:choose")
    if not url.endswith("/"):
        url += "/"
    return url


@register.simple_tag
def get_document_chosen_base_url():
    url = reverse("wagtaildocs_chooser:choose")
    if not url.endswith("/"):
        url += "/"
    return url


@register.simple_tag
def get_video_chosen_base_url():
    model = get_video_model()
    if model:
        url = model.snippet_viewset.chooser_viewset.get_url_name("choose")
        if not url.endswith("/"):
            url += "/"
        return url
    return ""


@register.simple_tag
def get_append_slash():
    return str(bool(settings.APPEND_SLASH)).lower()



from django.conf import settings
from django.templatetags.static import static
from django.utils.html import format_html
from wagtail import hooks
from wagtail.admin.menu import MenuItem
from wagtail.documents.wagtail_hooks import DocumentsSummaryItem
from wagtail.images.wagtail_hooks import ImagesSummaryItem
from wagtail.snippets.models import register_snippet

from wagtail_bynder import get_video_model
from wagtail_bynder.views.video import VideoViewSet


@hooks.register("construct_main_menu")
def hide_image_and_document_menu_items(request, menu_items):
    if domain := getattr(settings, "BYNDER_DOMAIN", ""):
        # Remove "Images" and "Documents" menu items
        menu_items[:] = [
            item for item in menu_items if item.name not in ("documents", "images")
        ]
        # Add a Bynder on in their place
        menu_items.insert(
            0,
            MenuItem(
                label="Manage assets",
                url=f"https://{domain}",
                classname="",
                icon_name="image",
                name="bynder",
                attrs={
                    "target": "_blank",
                    "title": "Manage assets in Bynder. Links open in a new tab.",
                },
                order=1000,
            ),
        )


@hooks.register("construct_homepage_summary_items")
def hide_image_and_document_summary_items(request, summary_items):
    if getattr(settings, "BYNDER_DOMAIN", ""):
        # Remove "Images" and "Documents" summary items
        summary_items[:] = [
            item
            for item in summary_items
            if not isinstance(item, ImagesSummaryItem | DocumentsSummaryItem)
        ]


@hooks.register("insert_editor_js")
def editor_js():
    if get_video_model():
        return format_html(
            """
            <script src="{0}"></script>
            """,
            static("bynder/js/video-chooser-modal.js"),
        )


if get_video_model():
    try:  # noqa: SIM105
        register_snippet(VideoViewSet)
    except Exception:  # noqa: S110
        pass



from django.utils.functional import cached_property
from wagtail.blocks import ChooserBlock


class VideoChooserBlock(ChooserBlock):
    @cached_property
    def target_model(self):
        from wagtail_bynder import get_video_model

        return get_video_model()

    @cached_property
    def widget(self):
        from wagtail_bynder.widgets import AdminVideoChooser

        return AdminVideoChooser()

    def render_basic(self, value, context=None):
        return ""

    class Meta:
        icon = "media"



import mimetypes
import os

from io import BytesIO

import requests

from asgiref.local import Local
from bynder_sdk import BynderClient
from django.conf import settings
from django.core.files.uploadedfile import InMemoryUploadedFile
from wagtail.models import Collection


_DEFAULT_COLLECTION = Local()


class DownloadedFile(BytesIO):
    name: str
    size: int
    content_type: str | None
    charset: str | None


def download_file(url: str) -> DownloadedFile:
    raw_bytes = requests.get(url, timeout=20).content
    f = DownloadedFile(raw_bytes)
    f.name = os.path.basename(url)
    f.size = len(raw_bytes)
    f.content_type, f.charset = mimetypes.guess_type(f.name)
    return f


def download_asset(url: str) -> InMemoryUploadedFile:
    f = download_file(url)
    return InMemoryUploadedFile(
        f,
        name=f.name,
        field_name="file",
        size=f.size,
        charset=f.charset,
        content_type=f.content_type,
    )


def filename_from_url(url: str) -> str:
    return os.path.basename(url)


def get_bynder_client() -> BynderClient:
    return BynderClient(
        domain=getattr(settings, "BYNDER_DOMAIN", ""),
        permanent_token=getattr(settings, "BYNDER_API_TOKEN", ""),
    )


def get_default_collection() -> Collection:
    """
    Return a Collection object that should be used as the default for images and
    documents created to represent Bynder assets.

    The result is cached for the current thread / asyncio task.
    """
    try:
        return _DEFAULT_COLLECTION.value
    except AttributeError:
        pass

    name = "Imported from Bynder"
    try:
        # Use existing collection
        collection = Collection.objects.get(name=name)
    except Collection.DoesNotExist:
        # Create a new one
        collection = Collection.get_first_root_node().add_child(name=name)

    # Cache result for the current thread / asyncio task
    _DEFAULT_COLLECTION.value = collection
    return collection



from typing import TYPE_CHECKING, Any

from django.utils.translation import gettext_lazy as _
from wagtail.images import get_image_model

from .base import BaseBynderSyncCommand


if TYPE_CHECKING:
    from wagtail_bynder.models import BynderAssetMixin


class Command(BaseBynderSyncCommand):
    help = _(
        "Update stale Wagtail image library items to reflect recent asset updates in Bynder."
    )
    model = get_image_model()
    bynder_asset_type: str = "image"
    page_size: int = 200

    def update_object(
        self, obj: "BynderAssetMixin", asset_data: dict[str, Any]
    ) -> None:
        """
        Overrides `BaseBynderSyncCommand.update_object()` to fetch the
        complete asset details before handing off to `obj.update_from_asset_data()`.
        (the API endpoint used by get_assets() does not include focal point data).
        """
        full_asset_data = self.bynder_client.asset_bank_client.media_info(
            asset_data["id"]
        )
        super().update_object(obj, full_asset_data)



from typing import TYPE_CHECKING

from django.conf import settings
from django.shortcuts import redirect

from wagtail_bynder.models import BynderAssetMixin
from wagtail_bynder.utils import get_bynder_client


if TYPE_CHECKING:
    from django.http import HttpRequest, HttpResponse


class BynderAssetCopyMixin:
    def create_object(self, asset_id: str) -> BynderAssetMixin:
        client = get_bynder_client()
        obj: BynderAssetMixin = self.model(bynder_id=asset_id)
        data = client.asset_bank_client.media_info(asset_id)
        obj.update_from_asset_data(data)
        obj.save()
        return obj

    def update_object(self, asset_id: str, obj: BynderAssetMixin) -> BynderAssetMixin:
        client = get_bynder_client()
        data = client.asset_bank_client.media_info(asset_id)
        if not obj.is_up_to_date(data):
            obj.update_from_asset_data(data)
            obj.save()
        return obj


class RedirectToBynderMixin:
    def setup(self, request, *args, **kwargs) -> None:
        if self.pk_url_kwarg != "pk":
            kwargs["pk"] = kwargs[self.pk_url_kwarg]
        super().setup(request, *args, **kwargs)

    def get(self, request: "HttpRequest", *args, **kwargs) -> "HttpResponse":
        self.object = self.get_object()
        if asset_id := self.object.bynder_id:
            return redirect(
                "https://" + settings.BYNDER_DOMAIN + f"/media/?mediaId={asset_id}"
            )
        return super().get(request, *args, **kwargs)

    def post(self, request: "HttpRequest", *args, **kwargs) -> "HttpResponse":
        self.object = self.get_object()
        if asset_id := self.object.bynder_id:
            return redirect(
                "https://" + settings.BYNDER_DOMAIN + f"/media/?mediaId={asset_id}"
            )
        return super().post(request, *args, **kwargs)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_video_model_string():
    """
    Get the dotted ``app.Model`` name for the video model as a string.
    """
    return getattr(settings, "BYNDER_VIDEO_MODEL", None)


def get_video_model():
    """
    Get the video model from the ``BYNDER_VIDEO_MODEL`` setting.
    """
    from django.apps import apps

    model_string = get_video_model_string()
    if model_string is None:
        return None

    try:
        return apps.get_model(model_string, require_ready=False)
    except ValueError as e:
        raise ImproperlyConfigured(
            "BYNDER_VIDEO_MODEL must be of the form 'app_label.Model'"
        ) from e
    except LookupError as e:
        raise ImproperlyConfigured(
            f"BYNDER_VIDEO_MODEL refers to model '{model_string}' that has not been installed"
        ) from e



from typing import TYPE_CHECKING

from django.conf import settings
from django.views.generic import UpdateView
from wagtail.documents import get_document_model
from wagtail.documents.views import chooser as chooser_views
from wagtail.documents.views.documents import DeleteView
from wagtail.documents.views.documents import edit as document_edit

from .mixins import BynderAssetCopyMixin, RedirectToBynderMixin


if TYPE_CHECKING:
    from django.http import HttpRequest, JsonResponse


class ClassBasedDocumentEditView(UpdateView):
    """
    A class-based view that mimics the behaviour of wagtail's function-based
    document edit view, and can be extended with view mixins.
    """

    # TODO: Use class from Wagtail once the documents app views are refactored
    model = get_document_model()
    pk_url_kwarg = "document_id"

    def get(self, request, *args, **kwargs):
        return document_edit(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        return document_edit(request, *args, **kwargs)


class DocumentEditView(RedirectToBynderMixin, ClassBasedDocumentEditView):
    pass


class DocumentDeleteView(RedirectToBynderMixin, DeleteView):
    pass


class DocumentChooseView(chooser_views.DocumentChooseView):
    choose_one_text = chooser_views.DocumentChooserViewSet.choose_one_text
    choose_another_text = chooser_views.DocumentChooserViewSet.choose_another_text
    page_title = choose_one_text
    permission_policy = chooser_views.DocumentChooserViewSet.permission_policy
    icon = chooser_views.DocumentChooserViewSet.icon
    template_name = "wagtailadmin/chooser/chooser-bynder.html"

    register_widget = False

    def __init__(self, *args, **kwargs):
        self.create_url_name = chooser_views.viewset.get_url_name("create")
        self.results_url_name = chooser_views.viewset.get_url_name("choose_results")
        self.collections = []
        super().__init__(*args, **kwargs)


class DocumentChosenView(chooser_views.DocumentChosenView, BynderAssetCopyMixin):
    model = get_document_model()

    def get(self, request: "HttpRequest", pk: str) -> "JsonResponse":
        try:
            obj = self.model.objects.get(bynder_id=pk)
        except self.model.DoesNotExist:
            obj = self.create_object(pk)
        else:
            if getattr(settings, "BYNDER_SYNC_EXISTING_DOCUMENTS_ON_CHOOSE", False):
                self.update_object(pk, obj)
        return self.get_chosen_response(obj)



from django.utils.translation import gettext_lazy as _
from wagtail.documents import get_document_model

from .base import BaseBynderRefreshCommand


class Command(BaseBynderRefreshCommand):
    help = _(
        "Update ALL Wagtail document library items to reflect the latest data from Bynder."
    )
    model = get_document_model()



from typing import TYPE_CHECKING

from django.conf import settings
from django.views.generic import UpdateView
from wagtail.images import get_image_model
from wagtail.images.views import chooser as chooser_views
from wagtail.images.views.images import DeleteView
from wagtail.images.views.images import edit as image_edit

from .mixins import BynderAssetCopyMixin, RedirectToBynderMixin


if TYPE_CHECKING:
    from django.http import HttpRequest, JsonResponse


class ClassBasedWagtailImageEditView(UpdateView):
    """
    A class-based view that mimics the behaviour of wagtail's function-based
    image edit view, and can be extended with view mixins.
    """

    # TODO: Use class from Wagtail once the image app views are refactored
    model = get_image_model()
    pk_url_kwarg = "image_id"

    def get(self, request, *args, **kwargs):
        return image_edit(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        return image_edit(request, *args, **kwargs)


class ImageEditView(RedirectToBynderMixin, ClassBasedWagtailImageEditView):
    pass


class ImageDeleteView(RedirectToBynderMixin, DeleteView):
    pass


class ImageChooseView(chooser_views.ImageChooseView):
    choose_one_text = chooser_views.ImageChooserViewSet.choose_one_text
    choose_another_text = chooser_views.ImageChooserViewSet.choose_another_text
    page_title = choose_one_text
    permission_policy = chooser_views.ImageChooserViewSet.permission_policy
    icon = chooser_views.ImageChooserViewSet.icon
    template_name = "wagtailadmin/chooser/chooser-bynder.html"

    def __init__(self, *args, **kwargs):
        self.create_url_name = chooser_views.viewset.get_url_name("create")
        self.results_url_name = chooser_views.viewset.get_url_name("choose_results")
        self.collections = []
        super().__init__(*args, **kwargs)


class ImageChosenView(chooser_views.ImageChosenView, BynderAssetCopyMixin):
    model = get_image_model()

    def get(self, request: "HttpRequest", pk: str) -> "JsonResponse":
        try:
            obj = self.model.objects.get(bynder_id=pk)
        except self.model.DoesNotExist:
            obj = self.create_object(pk)
        else:
            if getattr(settings, "BYNDER_SYNC_EXISTING_IMAGES_ON_CHOOSE", False):
                self.update_object(pk, obj)
        return self.get_chosen_response(obj)
